# Tutorial

In this tutorial, we are going to create the following mighty morphing JSON animation:

<code-movie-html language="json">
<pre>[]</pre>
<pre>["World"]</pre>
<pre>["Hello", "World"]</pre>
<pre>[
  "Hello",
  "World"
]</pre>
</code-movie-html>

[View on CodePen](https://codepen.io/SirPepe/pen/JjqpZae?editors=0010)

This is a somewhat modest example, but it encompasses everything you need to know - the process is essentially the same for all other programming languages and larger, more complex animations.

## Get the library

Code.Movie is essentially just a JavaScript library that turns strings into different strings. The library itself is written in TypeScript, but can also be used with plain JavaScript. It works in browsers as well as in server-side environments. You can use your preferred package manager to install [the library](https://www.npmjs.com/package/@codemovie/code-movie)...

```shell
$ npm install @codemovie/code-movie # adjust for use with yarn etc.
```

... but for this tutorial, we will use [the CDN jsdelivr](https://cdn.jsdelivr.net/npm/@codemovie/code-movie).

## Load the modules

Creating an animation involves two steps:

1. Loading the **language module** for the programming language that we want to use in out animation (JSON in our case)
2. Turning frames code into a giant **string of HTML** that renders the animation

To take care of step 2, there's just one function to load from the main module: `animateHTML()`. Assuming we are using a web browser and the CDN, we import the function from `@code-movie/code-movie` in a module:

```html
<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <title>Tutorial</title>
    <script type="module">
      import { animateHTML } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/index.js";
    </script>
  </head>
  <body></body>
</html>
```

## Load and configure the language

Code.Movie uses quite substantial parsing infrastructure that is different for every programming language. To improve performance, support for every language must be imported explicitly. For JSON this means importing `@codemovie/code-movie/languages/json`. Every language module exports a function as its sole default export. Calling the function returns an object that represents the language:

```javascript
import json from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/languages/json.js";
const language = json();
// "language" now contains everything needed for JSON support
```

Having to import every language manually is a tad annoying, but help to keep download and startup times in check. The function call also allows some languages to be configured in detail (like activating or deactivating support for JSX or type annotations in JavaScript).

## Rendering the animation

To create an animation, we first need a list of **keyframe objects** that our code animation steps through. This list is an array of objects like this:

```javascript
let keyframes = [
  { code: `[]` },
  { code: `["World"]` },
  { code: `["Hello", "World"]` },
  { code: `[\n  "Hello",\n  "World"\n]` },
];
```

This array then gets passed into the `animateHTML` function alongside our choice of tab size and the language we previously imported:

```javascript
import { animateHTML } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/index.js";
import json from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/languages/json.js";

const language = json();

let keyframes = [
  /* keyframes go here */
];

let html = animateHTML(keyframes, {
  tabSize: 2,
  language,
});
```

`animateHTML()` parses the code, computes the animation, and finally turns it into plain HTML and CSS. The resulting HTML string is _huge_ and contains everything a browser needs to display the animation.

The animation is advanced by switching between the classes `frame0`, `frame1`, `frameN` etc. on the element with the class `cm-animation` contained in the HTML, which by itself is not very user-friendly. We can add the custom element [`<code-movie-runtime>`](https://github.com/CodeMovie/code-movie-runtime) to our code to get a minimal user interface with equally minimal effort. Wrap the HTML returned by `toAnimationHTML` in `<code-movie-runtime>` tags, dump it into the page and we are done!

```javascript
import { animateHTML } from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/index.js";
import json from "https://cdn.jsdelivr.net/npm/@codemovie/code-movie/dist/languages/json.js";

const language = json();

let keyframes = [
  /* keyframes go here */
];

let html = animateHTML(keyframes, {
  tabSize: 2,
  language,
});

// Load the custom element
import "https://cdn.jsdelivr.net/npm/@codemovie/code-movie-runtime";

// Wrap the generated HTML and use it in the page!
document.body.innerHTML = `<code-movie-runtime controls keyframes="0 1 2 3">
  ${html}
</code-movie-runtime>`;
```

## Next steps

You can check out the [styling options](../styling/Readme.md) or read up on [API details](../reference/Readme.md) next.
